/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.impl.lab;

import com.fasterxml.jackson.core.JsonProcessingException;
import dk.sundhedsdatastyrelsen.cbs.dto.impl.Harness;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.lang.NonNull;

import java.util.stream.Stream;

import static dk.sundhedsdatastyrelsen.cbs.dto.impl.Harness.OBJECT_MAPPER;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.Harness.RANDOM_DATA;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.Harness.newAntigenResult;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.Harness.newPcrResult;
import static java.util.stream.Stream.generate;
import static org.junit.jupiter.api.Assertions.assertNotNull;

class LabResultImplTest {

    @MethodSource("result")
    @ParameterizedTest
    void serializeDeserialize(@NonNull final LabResult source) throws JsonProcessingException {
        final String serialized = OBJECT_MAPPER.writeValueAsString(source);
        assertNotNull(serialized);

        final LabResult deserialized = OBJECT_MAPPER.readValue(serialized, LabResultImpl.class);
        assertNotNull(deserialized);
        Harness.equals(source, deserialized);
    }

    @NonNull
    private static Stream<Arguments> result() {
        return generate(() -> RANDOM_DATA.nextBoolean() ? newPcrResult() : newAntigenResult())
            .limit(100)
            .map(Arguments::arguments);
    }

}
