/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.webservice.idws;

public enum IdwsUser {

    CPR_2401010010("2401010010", "PID:9208-2002-2-337576502433", "Trine Jansen"),
    CPR_2401010011("2401010011", "PID:9208-2002-2-878306162689", "Ellen Simonsen"),
    CPR_2401010012("2401010012", "PID:9208-2002-2-568409606447", "Tinja Gregersen"),
    CPR_2401010013("2401010013", "PID:9208-2002-2-257506556945", "Theiss Nielsen"),
    CPR_2401010014("2401010014", "PID:9208-2002-2-502257251507", "Tashina Mathiesen"),
    CPR_2401010015("2401010015", "PID:9208-2002-2-831965872884", "Thorborg Eriksen"),
    CPR_2401010016("2401010016", "PID:9208-2002-2-494976198983", "Thia Christoffersen"),
    CPR_2401010017("2401010017", "PID:9208-2002-2-604270903016", "Truels Holm"),
    CPR_2401010018("2401010018", "PID:9208-2002-2-652156092370", "Helle Frost Rasmussen"),
    CPR_3108732076("3108732076", "PID:9208-2002-2-098076399641", "Gitte Gregersen");

    private final String name;
    private final String cpr;
    private final String pid;

    IdwsUser(String cpr, String pid, String name) {
        this.name = name;
        this.cpr = cpr;
        this.pid = pid;
    }

    public String certificateInfo() {
        return "SubjectDN={C=DK,O=Ingen organisatorisk tilknytning,CN=" + name + ",Serial=" + pid + "},IssuerDN={C=ignored},CertSerial={1234}";
    }

    public String commonName() {
        return name;
    }

    public String cpr() {
        return cpr;
    }

    public String pid() {
        return pid;
    }
}
